@@ -19,7 +19,7 @@ import tornado
 from tornado import gen
 from tornado.ioloop import TimeoutError
 
-from distributed import (Nanny, Client, get_client, wait, default_client,
+from distributed import (Nanny, get_client, wait, default_client,
         get_worker, Reschedule)
 from distributed.compatibility import WINDOWS, cache_from_source
 from distributed.core import rpc
@@ -29,9 +29,8 @@ from distributed.metrics import time
 from distributed.worker import Worker, error_message, logger, TOTAL_MEMORY
 from distributed.utils import tmpfile, format_bytes
 from distributed.utils_test import (inc, mul, gen_cluster, div, dec,
-                                    slow, slowinc, gen_test, cluster,
-                                    captured_logger)
-from distributed.utils_test import loop, nodebug # noqa: F401
+                                    slow, slowinc, gen_test, captured_logger)
+from distributed.utils_test import client, loop, nodebug, cluster_fixture, s, a, b  # noqa: F401
 
 
 def test_worker_ncores():
@@ -400,7 +399,7 @@ def test_run_coroutine_dask_worker(c, s, a, b):
         yield gen.sleep(0.001)
         raise gen.Return(dask_worker.id)
 
-    response = yield c._run_coroutine(f)
+    response = yield c.run(f)
     assert response == {a.address: a.id, b.address: b.id}
 
 
@@ -705,21 +704,6 @@ def test_stop_doing_unnecessary_work(c, s, a, b):
 
 @gen_cluster(client=True, ncores=[('127.0.0.1', 1)])
 def test_priorities(c, s, w):
-    a = delayed(slowinc)(1, dask_key_name='a', delay=0.05)
-    b = delayed(slowinc)(2, dask_key_name='b', delay=0.05)
-    a1 = delayed(slowinc)(a, dask_key_name='a1', delay=0.05)
-    a2 = delayed(slowinc)(a1, dask_key_name='a2', delay=0.05)
-    b1 = delayed(slowinc)(b, dask_key_name='b1', delay=0.05)
-
-    z = delayed(add)(a2, b1)
-    future = yield c.compute(z)
-
-    log = [t for t in w.log if t[1] == 'executing' and t[2] == 'memory']
-    assert [t[0] for t in log[:5]] == ['a', 'b', 'a1', 'b1', 'a2']
-
-
-@gen_cluster(client=True, ncores=[('127.0.0.1', 1)])
-def test_priorities_2(c, s, w):
     values = []
     for i in range(10):
         a = delayed(slowinc)(i, dask_key_name='a-%d' % i, delay=0.01)
@@ -744,9 +728,11 @@ def test_priorities_2(c, s, w):
 @gen_cluster(client=True)
 def test_heartbeats(c, s, a, b):
     x = s.workers[a.address].last_seen
+    start = time()
     yield gen.sleep(a.periodic_callbacks['heartbeat'].callback_time / 1000 + 0.1)
-    y = s.workers[a.address].last_seen
-    assert x != y
+    while s.workers[a.address].last_seen == x:
+        yield gen.sleep(0.01)
+        assert time() < start + 2
     assert a.periodic_callbacks['heartbeat'].callback_time < 1000
 
 
@@ -755,8 +741,7 @@ def test_worker_dir(worker):
     with tmpfile() as fn:
         @gen_cluster(client=True, worker_kwargs={'local_dir': fn})
         def test_worker_dir(c, s, a, b):
-            directories = [info['local_directory']
-                           for info in s.worker_info.values()]
+            directories = [w.local_directory for w in s.workers.values()]
             assert all(d.startswith(fn) for d in directories)
             assert len(set(directories)) == 2  # distinct
 
@@ -832,7 +817,7 @@ def test_fail_write_many_to_disk(c, s, a):
 
 @gen_cluster()
 def test_pid(s, a, b):
-    assert s.worker_info[a.address]['pid'] == os.getpid()
+    assert s.workers[a.address].pid == os.getpid()
 
 
 @gen_cluster(client=True)
@@ -862,16 +847,14 @@ def test_get_client(c, s, a, b):
     assert a._client is a_client
 
 
-def test_get_client_sync(loop):
-    with cluster() as (s, [a, b]):
-        with Client(s['address'], loop=loop) as c:
-            def f(x):
-                cc = get_client()
-                future = cc.submit(inc, x)
-                return future.result()
+def test_get_client_sync(client):
+    def f(x):
+        cc = get_client()
+        future = cc.submit(inc, x)
+        return future.result()
 
-            future = c.submit(f, 10)
-            assert future.result() == 11
+    future = client.submit(f, 10)
+    assert future.result() == 11
 
 
 @gen_cluster(client=True)
@@ -883,24 +866,22 @@ def test_get_client_coroutine(c, s, a, b):
         result = yield future
         raise gen.Return(result)
 
-    results = yield c.run_coroutine(f)
+    results = yield c.run(f)
     assert results == {a.address: 11,
                        b.address: 11}
 
 
-def test_get_client_coroutine_sync(loop):
-    with cluster() as (s, [a, b]):
-        with Client(s['address'], loop=loop) as c:
-            @gen.coroutine
-            def f():
-                client = yield get_client()
-                future = client.submit(inc, 10)
-                result = yield future
-                raise gen.Return(result)
+def test_get_client_coroutine_sync(client, s, a, b):
+    @gen.coroutine
+    def f():
+        client = yield get_client()
+        future = client.submit(inc, 10)
+        result = yield future
+        raise gen.Return(result)
 
-            results = c.run_coroutine(f)
-            assert results == {a['address']: 11,
-                               b['address']: 11}
+    results = client.run(f)
+    assert results == {a['address']: 11,
+                       b['address']: 11}
 
 
 @gen_cluster()
@@ -1132,21 +1113,19 @@ def test_dict_data_if_no_spill_to_disk(s, w):
     assert type(w.data) is dict
 
 
-def test_get_worker_name(loop):
-    with cluster() as (s, [a, b]):
-        with Client(s['address'], loop=loop) as c:
-            def f():
-                get_client().submit(inc, 1).result()
+def test_get_worker_name(client):
+    def f():
+        get_client().submit(inc, 1).result()
 
-            c.run(f)
+    client.run(f)
 
-            def func(dask_scheduler):
-                return list(dask_scheduler.clients)
+    def func(dask_scheduler):
+        return list(dask_scheduler.clients)
 
-            start = time()
-            while not any('worker' in n for n in c.run_on_scheduler(func)):
-                sleep(0.1)
-                assert time() < start + 10
+    start = time()
+    while not any('worker' in n for n in client.run_on_scheduler(func)):
+        sleep(0.1)
+        assert time() < start + 10
 
 
 @gen_cluster(ncores=[('127.0.0.1', 1)],
@@ -1213,3 +1192,82 @@ def test_avoid_oversubscription(c, s, *workers):
 
     # Some other workers did some work
     assert len([w for w in workers if len(w.outgoing_transfer_log) > 0]) >= 3
+
+
+@gen_cluster(client=True, worker_kwargs={'metrics': {'my_port': lambda w: w.port}})
+def test_custom_metrics(c, s, a, b):
+    assert s.workers[a.address].metrics['my_port'] == a.port
+    assert s.workers[b.address].metrics['my_port'] == b.port
+
+
+@gen_cluster(client=True)
+def test_register_worker_callbacks(c, s, a, b):
+    #preload function to run
+    def mystartup(dask_worker):
+        dask_worker.init_variable = 1
+
+    def mystartup2():
+        import os
+        os.environ['MY_ENV_VALUE'] = 'WORKER_ENV_VALUE'
+        return "Env set."
+
+    #Check that preload function has been run
+    def test_import(dask_worker):
+        return hasattr(dask_worker, 'init_variable')
+        #       and dask_worker.init_variable == 1
+
+    def test_startup2():
+        import os
+        return os.getenv('MY_ENV_VALUE', None) == 'WORKER_ENV_VALUE'
+
+    # Nothing has been run yet
+    assert len(s.worker_setups) == 0
+    result = yield c.run(test_import)
+    assert list(result.values()) == [False] * 2
+    result = yield c.run(test_startup2)
+    assert list(result.values()) == [False] * 2
+
+    # Start a worker and check that startup is not run
+    worker = Worker(s.address, loop=s.loop)
+    yield worker._start()
+    result = yield c.run(test_import, workers=[worker.address])
+    assert list(result.values()) == [False]
+    yield worker._close()
+
+    # Add a preload function
+    response = yield c.register_worker_callbacks(setup=mystartup)
+    assert len(response) == 2
+    assert len(s.worker_setups) == 1
+
+    # Check it has been ran on existing worker
+    result = yield c.run(test_import)
+    assert list(result.values()) == [True] * 2
+
+    # Start a worker and check it is ran on it
+    worker = Worker(s.address, loop=s.loop)
+    yield worker._start()
+    result = yield c.run(test_import, workers=[worker.address])
+    assert list(result.values()) == [True]
+    yield worker._close()
+
+    # Register another preload function
+    response = yield c.register_worker_callbacks(setup=mystartup2)
+    assert len(response) == 2
+    assert len(s.worker_setups) == 2
+
+    # Check it has been run
+    result = yield c.run(test_startup2)
+    assert list(result.values()) == [True] * 2
+
+    # Start a worker and check it is ran on it
+    worker = Worker(s.address, loop=s.loop)
+    yield worker._start()
+    result = yield c.run(test_import, workers=[worker.address])
+    assert list(result.values()) == [True]
+    result = yield c.run(test_startup2, workers=[worker.address])
+    assert list(result.values()) == [True]
+    yield worker._close()
+
+    # Final exception test
+    with pytest.raises(ZeroDivisionError):
+        yield c.register_worker_callbacks(setup=lambda: 1 / 0)
